{
  "type": "record",
  "Aspect": {
    "name": "dataHubPersonaInfo"
  },
  "name": "DataHubPersonaInfo",
  "namespace": "com.linkedin.pegasus2avro.persona",
  "fields": [
    {
      "java": {
        "class": "com.linkedin.pegasus2avro.common.urn.Urn"
      },
      "type": [
        "null",
        "string"
      ],
      "name": "view",
      "default": null,
      "doc": "The urn of the search view associated with the persona.",
      "Urn": "Urn"
    }
  ],
  "doc": "Placeholder aspect for persona type info"
}