# mypy: ignore-errors
# flake8: noqa

# This file is autogenerated by /metadata-ingestion/scripts/avro_codegen.py
# Do not modify manually!

# pylint: skip-file
# fmt: off
# isort: skip_file
from .....schema_classes import DataHubActionConfigClass
from .....schema_classes import DataHubActionInfoClass
from .....schema_classes import DataHubActionRequestSourceClass
from .....schema_classes import DataHubActionSourceClass
from .....schema_classes import DataHubActionStageStatusClass
from .....schema_classes import DataHubActionStageStatusCodeClass
from .....schema_classes import DataHubActionStateClass
from .....schema_classes import DataHubActionStatusClass


DataHubActionConfig = DataHubActionConfigClass
DataHubActionInfo = DataHubActionInfoClass
DataHubActionRequestSource = DataHubActionRequestSourceClass
DataHubActionSource = DataHubActionSourceClass
DataHubActionStageStatus = DataHubActionStageStatusClass
DataHubActionStageStatusCode = DataHubActionStageStatusCodeClass
DataHubActionState = DataHubActionStateClass
DataHubActionStatus = DataHubActionStatusClass

# fmt: on
